#ifndef CONFIGSERVICE_H
#define CONFIGSERVICE_H

#include <ESP8266WiFi.h>

/** Number of connections, limited by the framework. */
#ifndef MAX_TCP_CONNECTIONS
#define MAX_TCP_CONNECTIONS 5
#endif


/** Config settings in EEProm (at address 0) */
struct EEPromData
{
    char webserver[100];
    // here you could add additional configuration settings for your application
    // if you do so, you also need to edit ConfigService::ConnectionHandler::run()
};
extern EEPromData eePromData;


/**
 * Provides a minimal HTTP server for configuration settings.
 * During the first n minutes, this services enables the soft AP
 * and can be accessed on http://192.68.4.1 . It displays
 * a HTML form where the user enters the SSID and password off his
 * AP.
 */
class ConfigService 
{
public:
    
    /** 
     * Constructor.
     * @param port The IP port to listen on (e.g. 80) 
     */
    ConfigService(uint16_t port);
    
    /**
     * Enables the soft AP and starts listening for HTTP connections.
     * @param minutes Disable the service after n minutes (0=never disable).
     * @param apName Name of the temporary access point.
     * @param password Password of the temporary access point, must be an empty string or min. 8 characters.
     */
    void begin(unsigned int minutes, const char* apName, const char* password);
    
    /** Stop listening for HTTP connections. */
    void stop();
    
    /** Must be called by the main loop to execute the service */
    void run();  
            
private:
    
    /** Processes one connection */
    class ConnectionHandler
    {
    public:       
        /** Constructor */
        ConnectionHandler();
        
        /** Set up for a new connection */
        void setup(WiFiClient& client);
               
        /** Must be called to process the connection */
        void run();
        
        /** Reference to the connection object of the framework */
        WiFiClient client;
        
        /** Buffer for HTTP requests */
        char buffer[200];
        
        /** State of the connection handler */
        enum {WAIT_REQUEST, WAIT_EMPTY_LINE, PROCESS_REQUEST} state;
    };
     
    /** Listens for incoming connections */
    WiFiServer server;
    
    /** One object for each possible connection */
    ConnectionHandler connections[MAX_TCP_CONNECTIONS];
    
    /** Number of the current connection */
    int connIndex;
    
    /** Disable the service at the given time */
    unsigned long stopAt;
    
    /** The port number that this services listens on */
    uint16_t port;        
};


#endif /* CONFIGSERVICE_H */

