#ifndef UTILS_H
#define UTILS_H

#include "Stream.h"

/**
 * Append characters from a stream to the buffer until the terminator has been found.
 * In case of buffer overflow, read until the terminator but skip all characters that 
 * do not fit into the buffer.
 * @param source The source stream.
 * @param buffer The target buffer, must be terminated with '\0'.
 * @param bufSize Size of the buffer.
 * @param terminator The last character that shall be read from the stream, e.g. '\n'.
 * @return True if the terminator has been reached, false if the stream did not contain the terminator.
 */
bool appendUntil(Stream& source, char* buffer, int bufSize, char terminator);

/**
 * Read and ignore all characters from stream until the terminator has been found.
 * @param source The source stream.
 * @param terminator The last character that shall be read from the stream, e.g. '\n'.
 * @return True if the terminator has been reached, false if the stream did not contain the terminator.
 */
bool flushUntil(Stream& source, char terminator);


/**
 * Decode an encoded URL parameter value.
 * @param buffer Gets filled with the result.
 * @param bufSize Size of the buffer.
 * @param source Pointer to the begin of the URL parameter value.
 */
void urlDecode(char* buffer, int bufSize, const char* source);


#endif /* UTILS_H */

